#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['drf_service_layer']

package_data = \
{'': ['*']}

install_requires = \
['django', 'djangorestframework']

extras_require = \
{'dev': ['pytest >=6.2.4,<7.0.0',
         'pytest-django ==4.4.0',
         'pytest-cov >=2.12.0,<3.0.0',
         'flake8 >=3.8.3,<4.0.0',
         'black ==20.8b1',
         'isort >=5.0.6,<6.0.0']}

setup(name='drf-service-layer',
      version='0.0.10',
      description='Simple package which supports service-layered design for Django REST Framework.',
      author=None,
      author_email='qu3vipon <qu3vipon@gmail.com>',
      url=None,
      packages=packages,
      package_data=package_data,
      install_requires=install_requires,
      extras_require=extras_require,
      python_requires='>=3.7',
     )
